@@ -11,12 +11,17 @@
 /*****************************************************************************/
 
 /*
- * dwgfuzz.c: afl++ fuzzing for all in- and exporters. Just not the seperate ones.
- *            Also useful for debugging the fuzzers.
- * AFL_DONT_OPTIMIZE=1 AFL_LLVM_INSTRIM=1 AFL_USE_ASAN=1 make -C examples dwgfuzz V=1
- * AFL_DEBUG=15 AFL_DEBUG_CHILD_OUTPUT=1 gdb --args afl-fuzz -m none -i ../.fuzz-in-dxf/
+ * dwgfuzz.c: afl++/honggfuzz fuzzing for all in- and exporters. Just not the seperate ones.
+ *
+ * Also usable like:
+    ../configure --disable-shared --disable-bindings CC=hfuzz-clang CFLAGS='-O2 -g -fsanitize=address,undefined -fno-omit-frame-pointer -I/usr/local/include' && make -C src && make -C examples dwgfuzz
+    honggfuzz -i ../.fuzz-in-dxf -- examples/dwgfuzz -indxf ___FILE___
+
+ * Also useful for debugging the fuzzers.
+    AFL_DONT_OPTIMIZE=1 AFL_LLVM_INSTRIM=1 AFL_USE_ASAN=1 make -C examples dwgfuzz V=1
+    AFL_DEBUG=15 AFL_DEBUG_CHILD_OUTPUT=1 gdb --args afl-fuzz -m none -i ../.fuzz-in-dxf/
       -o .fuzz-out/ examples/dwgfuzz
- * (gdb) set follow-fork-mode child
+    (gdb) set follow-fork-mode child
  * written by Reini Urban
  */
 
@@ -134,6 +139,8 @@ help (void)
   exit (0);
 }
 
+// for LLVMFuzzerTestOneInput see llvmfuzz.c
+
 int
 main (int argc, char *argv[])
 {
@@ -573,13 +580,13 @@ int dwg_fuzz_dat (Dwg_Data **restrict dwgp, Bit_Chain *restrict dat)
       else if (SSCANF_S (p, #var "." FMT_NAME " = " FMT_ANY, &s1[0] SZ, &text[0] SZ)) \
         dwg_dynapi_entity_set_value (var, #name, s1, text, 1)
 
-      if (sscanf (p, "line (%lf %lf %lf) (%lf %lf %lf)", &pt1.x, &pt1.y,
+      if (SSCANF_S (p, "line (%lf %lf %lf) (%lf %lf %lf)", &pt1.x, &pt1.y,
                   &pt1.z, &pt2.x, &pt2.y, &pt2.z))
         dwg_add_LINE (hdr, &pt1, &pt2);
-      else if (sscanf (p, "ray (%lf %lf %lf) (%lf %lf %lf)", &pt1.x, &pt1.y,
+      else if (SSCANF_S (p, "ray (%lf %lf %lf) (%lf %lf %lf)", &pt1.x, &pt1.y,
                   &pt1.z, &pt2.x, &pt2.y, &pt2.z))
         dwg_add_RAY (hdr, &pt1, &pt2);
-      else if (sscanf (p, "xline (%lf %lf %lf) (%lf %lf %lf)", &pt1.x, &pt1.y,
+      else if (SSCANF_S (p, "xline (%lf %lf %lf) (%lf %lf %lf)", &pt1.x, &pt1.y,
                   &pt1.z, &pt2.x, &pt2.y, &pt2.z))
         dwg_add_XLINE (hdr, &pt1, &pt2);
       else if (SSCANF_S (p, "text " FMT_ANY " (%lf %lf %lf) %lf", &text[0] SZ, &pt1.x,
@@ -593,73 +600,73 @@ int dwg_fuzz_dat (Dwg_Data **restrict dwgp, Bit_Chain *restrict dat)
         dwg_add_BLOCK (hdr, text);
       else if (memBEGINc (p, "endblk\n"))
         dwg_add_ENDBLK (hdr);
-      else if (sscanf (p, "insert (%lf %lf %lf) " FMT_TBL " %lf %lf %lf %lf",
+      else if (SSCANF_S (p, "insert (%lf %lf %lf) " FMT_TBL " %lf %lf %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &text[0] SZ, &scale.x, &scale.y,
                        &scale.z, &rot))
         dwg_add_INSERT (hdr, &pt1, text, scale.x, scale.y, scale.z, deg2rad (rot));
-      else if (sscanf (p,
+      else if (SSCANF_S (p,
                        "minsert (%lf %lf %lf) " FMT_TBL " %lf %lf %lf %lf %d %d "
                        "%lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &text[0] SZ, &scale.x, &scale.y,
                        &scale.z, &rot, &i1, &i2, &f1, &f2))
         dwg_add_MINSERT (hdr, &pt1, text, scale.x, scale.y, scale.z, deg2rad (rot), i1,
                          i2, f1, f2);
-      else if (sscanf (p, "point (%lf %lf %lf)", &pt1.x, &pt1.y, &pt1.z))
+      else if (SSCANF_S (p, "point (%lf %lf %lf)", &pt1.x, &pt1.y, &pt1.z))
         dwg_add_POINT (hdr, &pt1);
-      else if (sscanf (p, "circle (%lf %lf %lf) %lf", &pt1.x, &pt1.y, &pt1.z,
+      else if (SSCANF_S (p, "circle (%lf %lf %lf) %lf", &pt1.x, &pt1.y, &pt1.z,
                        &f1))
         dwg_add_CIRCLE (hdr, &pt1, f1);
-      else if (sscanf (p, "arc (%lf %lf %lf) %lf %lf %lf", &pt1.x, &pt1.y,
+      else if (SSCANF_S (p, "arc (%lf %lf %lf) %lf %lf %lf", &pt1.x, &pt1.y,
                        &pt1.z, &f1, &f2, &height))
         dwg_add_ARC (hdr, &pt1, f1, f2, height);
-      else if (sscanf (p, "dimension_aligned (%lf %lf %lf) (%lf %lf %lf) (%lf "
+      else if (SSCANF_S (p, "dimension_aligned (%lf %lf %lf) (%lf %lf %lf) (%lf "
                        "%lf %lf)",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &pt3.x,
                        &pt3.y, &pt3.z))
         dwg_add_DIMENSION_ALIGNED (hdr, &pt1, &pt2, &pt3);
-      else if (sscanf (p, "dimension_linear (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
+      else if (SSCANF_S (p, "dimension_linear (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
                        "%lf) %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &pt3.x,
                        &pt3.y, &pt3.z, &rot))
         dwg_add_DIMENSION_LINEAR (hdr, &pt1, &pt2, &pt3, deg2rad (rot));
-      else if (sscanf (p, "dimension_ang2ln (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
+      else if (SSCANF_S (p, "dimension_ang2ln (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
                        "%lf) (%lf %lf %lf)",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &pt3.x,
                        &pt3.y, &pt3.z, &pt4.x, &pt4.y, &pt4.z))
         dwg_add_DIMENSION_ANG2LN (hdr, &pt1, &pt2, &pt3, &pt4);
-      else if (sscanf (p, "dimension_ang3pt (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
+      else if (SSCANF_S (p, "dimension_ang3pt (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
                        "%lf) (%lf %lf %lf)",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &pt3.x,
                        &pt3.y, &pt3.z, &pt4.x, &pt4.y, &pt4.z))
         dwg_add_DIMENSION_ANG3PT (hdr, &pt1, &pt2, &pt3, &pt4);
-      else if (sscanf (p, "dimension_diameter (%lf %lf %lf) (%lf %lf %lf) %lf",
+      else if (SSCANF_S (p, "dimension_diameter (%lf %lf %lf) (%lf %lf %lf) %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &len))
         dwg_add_DIMENSION_DIAMETER (hdr, &pt1, &pt2, len);
-      else if (sscanf (p, "dimension_ordinate (%lf %lf %lf) (%lf %lf %lf) %d",
+      else if (SSCANF_S (p, "dimension_ordinate (%lf %lf %lf) (%lf %lf %lf) %d",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &i1))
         dwg_add_DIMENSION_ORDINATE (hdr, &pt1, &pt2, i1 ? true : false);
-      else if (sscanf (p, "dimension_radius (%lf %lf %lf) (%lf %lf %lf) %lf",
+      else if (SSCANF_S (p, "dimension_radius (%lf %lf %lf) (%lf %lf %lf) %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &len))
         dwg_add_DIMENSION_RADIUS (hdr, &pt1, &pt2, len);
-      else if (sscanf (p, "3dface (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
+      else if (SSCANF_S (p, "3dface (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
                        "%lf) (%lf %lf %lf)",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &pt3.x,
                        &pt3.y, &pt3.z, &pt4.x, &pt4.y, &pt4.z))
         dwg_add_3DFACE (hdr, &pt1, &pt2, &pt3, &pt4);
-      else if (sscanf (p,"3dface (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
+      else if (SSCANF_S (p,"3dface (%lf %lf %lf) (%lf %lf %lf) (%lf %lf "
                        "%lf)",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &pt3.x,
                        &pt3.y, &pt3.z))
         dwg_add_3DFACE (hdr, &pt1, &pt2, &pt3, NULL);
-      else if (sscanf (p, "solid (%lf %lf %lf) (%lf %lf) (%lf %lf)  (%lf %lf)",
+      else if (SSCANF_S (p, "solid (%lf %lf %lf) (%lf %lf) (%lf %lf)  (%lf %lf)",
                        &pt1.x, &pt1.y, &pt1.z, &p2.x, &p2.y, &p3.x,
                        &p3.y, &p4.x, &p4.y))
         dwg_add_SOLID (hdr, &pt1, &p2, &p3, &p4);
-      else if (sscanf (p, "trace (%lf %lf %lf) (%lf %lf) (%lf %lf)  (%lf %lf)",
+      else if (SSCANF_S (p, "trace (%lf %lf %lf) (%lf %lf) (%lf %lf)  (%lf %lf)",
                        &pt1.x, &pt1.y, &pt1.z, &p2.x, &p2.y, &p3.x,
                        &p3.y, &p4.x, &p4.y))
         dwg_add_TRACE (hdr, &pt1, &p2, &p3, &p4);
-      else if (sscanf (p, "polyline_2d %d ((%lf %lf)", &i1, &pt1.x, &pt1.y))
+      else if (SSCANF_S (p, "polyline_2d %d ((%lf %lf)", &i1, &pt1.x, &pt1.y))
         {
           dwg_point_2d *pts = scan_pts2d (i1, &p);
           if (i1 && pts)
@@ -668,7 +675,7 @@ int dwg_fuzz_dat (Dwg_Data **restrict dwgp, Bit_Chain *restrict dat)
               free (pts);
             }
         }
-      else if (sscanf (p, "polyline_3d %d ((%lf %lf %lf)", &i1, &pt1.x, &pt1.y, &pt1.z))
+      else if (SSCANF_S (p, "polyline_3d %d ((%lf %lf %lf)", &i1, &pt1.x, &pt1.y, &pt1.z))
         {
           dwg_point_3d *pts = scan_pts3d (i1, &p);
           if (i1 && pts)
@@ -677,7 +684,7 @@ int dwg_fuzz_dat (Dwg_Data **restrict dwgp, Bit_Chain *restrict dat)
               free (pts);
             }
         }
-      else if (sscanf (p, "polyline_mesh %d %d ((%lf %lf %lf)", &i1, &i2, &pt1.x, &pt1.y, &pt1.z))
+      else if (SSCANF_S (p, "polyline_mesh %d %d ((%lf %lf %lf)", &i1, &i2, &pt1.x, &pt1.y, &pt1.z))
         {
           dwg_point_3d *pts = scan_pts3d (i1 * i2, &p);
           if (i1 && i2 && pts)
@@ -688,18 +695,18 @@ int dwg_fuzz_dat (Dwg_Data **restrict dwgp, Bit_Chain *restrict dat)
         }
       else if (SSCANF_S (p, "dictionary " FMT_TBL " " FMT_TBL " %u", &text[0] SZ, &s1[0] SZ, &u))
         dictionary = dwg_add_DICTIONARY (dwg, text, s1, (unsigned long)u);
-      else if (dictionary && sscanf (p, "xrecord dictionary " FMT_TBL, &text[0] SZ))
+      else if (dictionary && SSCANF_S (p, "xrecord dictionary " FMT_TBL, &text[0] SZ))
         xrecord = dwg_add_XRECORD (dictionary, text);
       else if (SSCANF_S (p, "shape " FMT_PATH " (%lf %lf %lf) %lf %lf",
                        &text[0] SZ, &pt1.x, &pt1.y, &pt1.z, &scale.x, &rot))
         dwg_add_SHAPE (hdr, text, &pt1, scale.x, deg2rad (rot));
-      else if (sscanf (p, "viewport " FMT_TBL, &text[0] SZ))
+      else if (SSCANF_S (p, "viewport " FMT_TBL, &text[0] SZ))
         viewport = dwg_add_VIEWPORT (hdr, text);
       else SET_ENT (viewport, VIEWPORT);
-      else if (sscanf (p, "ellipse (%lf %lf %lf) %lf %lf",
+      else if (SSCANF_S (p, "ellipse (%lf %lf %lf) %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &f1, &f2))
         dwg_add_ELLIPSE (hdr, &pt1, f1, f2);
-      else if (sscanf (p, "spline %d ((%lf %lf %lf)", &i1, &pt1.x, &pt1.y, &pt1.z))
+      else if (SSCANF_S (p, "spline %d ((%lf %lf %lf)", &i1, &pt1.x, &pt1.y, &pt1.z))
         {
           dwg_point_3d *fitpts = scan_pts3d (i1, &p);
           if (i1 && fitpts && sscanf (p, ") (%lf %lf %lf) (%lf %lf %lf)", &pt2.x,
@@ -740,25 +747,25 @@ int dwg_fuzz_dat (Dwg_Data **restrict dwgp, Bit_Chain *restrict dat)
           if (!error)
             layout = dwg_add_LAYOUT (obj, text, s1);
         }
-      else if (sscanf (p, "torus (%lf %lf %lf) (%lf %lf %lf) %lf %lf",
+      else if (SSCANF_S (p, "torus (%lf %lf %lf) (%lf %lf %lf) %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &f1, &f2))
         dwg_add_TORUS (hdr, &pt1, &pt2, f1, f2);
-      else if (sscanf (p, "sphere (%lf %lf %lf) (%lf %lf %lf) %lf",
+      else if (SSCANF_S (p, "sphere (%lf %lf %lf) (%lf %lf %lf) %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &f1))
         dwg_add_SPHERE (hdr, &pt1, &pt2, f1);
-      else if (sscanf (p, "cylinder (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf %lf",
+      else if (SSCANF_S (p, "cylinder (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &height, &f1, &f2, &len))
         dwg_add_CYLINDER (hdr, &pt1, &pt2, height, f1, f2, len);
-      else if (sscanf (p, "cone (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf %lf",
+      else if (SSCANF_S (p, "cone (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &height, &f1, &f2, &len))
         dwg_add_CONE (hdr, &pt1, &pt2, height, f1, f2, len);
-      else if (sscanf (p, "wedge (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf",
+      else if (SSCANF_S (p, "wedge (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &len, &f1, &height))
         dwg_add_WEDGE (hdr, &pt1, &pt2, len, f1, height);
-      else if (sscanf (p, "box (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf",
+      else if (SSCANF_S (p, "box (%lf %lf %lf) (%lf %lf %lf) %lf %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &len, &f1, &height))
         dwg_add_BOX (hdr, &pt1, &pt2, len, f1, height);
-      else if (sscanf (p, "pyramid (%lf %lf %lf) (%lf %lf %lf) %lf %d %lf %lf",
+      else if (SSCANF_S (p, "pyramid (%lf %lf %lf) (%lf %lf %lf) %lf %d %lf %lf",
                        &pt1.x, &pt1.y, &pt1.z, &pt2.x, &pt2.y, &pt2.z, &height, &i1, &f1, &f2))
         dwg_add_PYRAMID (hdr, &pt1, &pt2, height, i1, f1, f2);
       else if (SSCANF_S (p, "HEADER." FMT_NAME " = %d", &s1[0] SZ, &i1))
