@@ -74,12 +74,12 @@ help (void)
 /* handles r2007 wide chars (=> utf8) */
 static int
 set_info (PSDoc *restrict ps, Dwg_Data *restrict dwg, const char *restrict key,
-          char *restrict text)
+          BITCODE_TU restrict text)
 {
   int ret = 0;
-  if (dwg->header.version >= R_2007 && text)
+  if (text)
     {
-      char *u8 = bit_convert_TU ((BITCODE_TU)text);
+      char *u8 = bit_convert_TU (text);
       if (u8 && strlen (u8))
         {
           PS_set_info (ps, key, u8);
@@ -88,15 +88,7 @@ set_info (PSDoc *restrict ps, Dwg_Data *restrict dwg, const char *restrict key,
       if (u8)
         free (u8);
     }
-  else
-    {
-      if (text && strlen (text))
-        {
-          PS_set_info (ps, key, text);
-          ret = 1;
-        }
-    }
-  return 1;
+  return ret;
 }
 
 static void
@@ -143,7 +135,7 @@ create_postscript (Dwg_Data *dwg, char *output)
   PS_scale (ps, (float)scale, (float)scale);
   PS_translate (ps, (float)-dwg_model_x_min (dwg),
                 (float)-dwg_model_y_min (dwg));
-  if (dwg->opts & 0xf)
+  if (dwg->opts & DWG_OPTS_LOGLEVEL)
     {
       fprintf (stderr, "Limits: %f, %f\n", dx, dy);
       fprintf (stderr, "Scale: %f (%f, %f)\n", scale, scale_x, scale_y);
@@ -164,8 +156,6 @@ create_postscript (Dwg_Data *dwg, char *output)
   for (i = 0; i < dwg->num_objects; i++)
     {
       Dwg_Object *obj = &dwg->object[i];
-      if (obj->supertype == DWG_SUPERTYPE_UNKNOWN) // unknown
-        continue;
       if (obj->type == DWG_SUPERTYPE_OBJECT) // no entity
         continue;
       // if (obj->tio.entity->entity_mode == 0) // belongs to block
