@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009-2010,2018-2020 Free Software Foundation, Inc.         */
+/*  Copyright (C) 2009-2010,2018-2021 Free Software Foundation, Inc.         */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -109,7 +109,7 @@ dat_read_file (Bit_Chain *restrict dat, FILE *restrict fp,
       dat->size = 0;
       return DWG_ERR_IOERROR;
     }
-  dat->chain[dat->size] = '\0';  // ensure zero-terminatation for strstr
+  dat->chain[dat->size] = '\0';  // ensure zero-termination for strstr, strtol, ...
   return 0;
 }
 
@@ -135,6 +135,7 @@ dat_read_size (Bit_Chain *restrict dat)
       dat->chain = NULL;
       return DWG_ERR_IOERROR;
     }
+  dat->chain[dat->size] = '\0';  // ensure zero-termination
   return 0;
 }
 
@@ -177,10 +178,10 @@ dat_read_stream (Bit_Chain *restrict dat, FILE *restrict fp)
   // clear the slack and realloc
   size = dat->size & 0xfff;
   if (size)
-    {
-      memset (&dat->chain[dat->size], 0, 0xfff - size);
-      dat->chain = (unsigned char *)realloc (dat->chain, dat->size);
-    }
+    memset (&dat->chain[dat->size], 0, 0xfff - size);
+  dat->chain = (unsigned char *)realloc (dat->chain, dat->size + 1);
+  // ensure NULL termination, for sscanf, strtol and friends.
+  dat->chain[dat->size] = '\0';
   return 0;
 }
 
@@ -343,15 +344,19 @@ dxf_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
       dat.size = 0;
       return DWG_ERR_IOERROR;
     }
-  // properly end the buffer for strtol()/... readers
-  dat.chain[size] = '\n';
-  dat.chain[size + 1] = '\0';
   if (size < 256)
     {
       LOG_ERROR ("File %s too small, %lu byte.\n", filename,
                  (long unsigned int)size)
       return DWG_ERR_IOERROR;
     }
+  // properly end the buffer for strtol()/... readers
+  if (dat.chain[size-1] != '\n')
+    {
+      dat.chain[size] = '\n';
+      dat.size++;
+    }
+  dat.chain[size] = '\0';
 
   /* Fail on DWG */
   if (!memcmp (dat.chain, "AC10", 4) ||
@@ -552,6 +557,11 @@ dwg_bmp (const Dwg_Data *restrict dwg, BITCODE_RL *restrict size)
           *size = bit_read_RL (&dat);
           found = 1;
           LOG_INFO ("\t\tBMP size: %i [RL]\n", *size)
+          if (*size > (dat.size - 4))
+            {
+              LOG_ERROR ("BMP thumbnail overflow > %ld", dat.size - 4);
+              return NULL;
+            }
         }
       else if (type == 3)
         {
@@ -571,7 +581,13 @@ dwg_bmp (const Dwg_Data *restrict dwg, BITCODE_RL *restrict size)
     }
   dat.byte += header_size;
   if (*size)
-    LOG_TRACE ("BMP offset: %lu\n", dat.byte)
+    LOG_TRACE ("BMP offset: %lu\n", dat.byte);
+  if (dat.byte > dat.size)
+    {
+      *size = 0;
+      LOG_ERROR ("Preview overflow");
+      return NULL;
+    }
 
   if (*size > 0)
     return (dat.chain + dat.byte);
