@@ -252,6 +252,9 @@
         }                                                                     \
     }
 #endif
+#ifndef FIELD_VECTOR_T1
+#  define FIELD_VECTOR_T1(nam, type, size, dxf) FIELD_VECTOR_T(nam, type, size, dxf)
+#endif
 #ifndef SUB_FIELD_VECTOR_TYPESIZE
 #  define SUB_FIELD_VECTOR_TYPESIZE(o, nam, size, typesize, dxf)              \
   if (_obj->o.size && _obj->o.nam)                                            \
@@ -529,11 +532,11 @@
     {                                                                         \
       if (strcmp (#acdbname, "Layer") == 0)                                   \
         {                                                                     \
-        FIELD_CAST (flag, RC, RS, 70);                                        \
+          FIELD_CAST (flag, RC, RS, 70);                                      \
         }                                                                     \
       else                                                                    \
         {                                                                     \
-          FIELD_RC (flag, 70);                                                \
+          FIELD_CAST (flag, RC, RC, 70);                                      \
         }                                                                     \
       FIELD_TFv (name, 32, 2);                                                \
       FIELD_RS (used, 0);                                                     \
@@ -555,8 +558,8 @@
           FIELD_VALUE (is_xref_dep) = 1;                                      \
       }                                                                       \
       FIELD_HANDLE (xref, 5, 0); /* NULLHDL without is_xref_dep */            \
-      FIELD_VALUE (flag) |= FIELD_VALUE (is_xref_dep) << 4                    \
-                          | FIELD_VALUE (is_xref_ref) << 6;                   \
+      FIELD_VALUE (flag)                                                      \
+          |= FIELD_VALUE (is_xref_dep) << 4 | FIELD_VALUE (is_xref_ref) << 6; \
     }                                                                         \
     RESET_VER
 #endif
