@@ -708,7 +708,7 @@ main (int argc, char const *argv[])
     color.name = (char *)"Some color";
     color.book_name = (char *)"DIC(3) Catalog";
     bit_write_CMC (&bitchain, &bitchain, &color);
-    if (bitchain.byte == 133 && bitchain.bit == 0)
+    if (bitchain.byte == 131 && bitchain.bit == 0)
       pass ();
     else
       fail ("bit_write_CMC (r2004) @%ld.%d", bitchain.byte, bitchain.bit);
@@ -742,7 +742,7 @@ main (int argc, char const *argv[])
   bitchain.byte = 0;
   {
     int ret = bit_search_sentinel (&bitchain, sentinel);
-    if (bitchain.byte == 150)
+    if (bitchain.byte == 148)
       pass ();
     else
       {
@@ -753,7 +753,7 @@ main (int argc, char const *argv[])
   {
     unsigned int check
         = bit_calc_CRC (0xC0C1, (unsigned char *)bitchain.chain, 124L);
-    if (check == 0xFEC1)
+    if (check == 0x43B8)
       pass ();
     else
       fail ("bit_calc_CRC %04X", check);
